from semfio_mist.logger import logger, logger_engine
from semfio_mist.config import Config
from semfio_mist.mist_api import API


class WLAN:
    """Mist WLAN Object
    """

    wlan_id: str = None
    ssid: str
    site_id: str
    api: API
    band: str = None
    interface: str = None
    hostname_ie: str = None
    auth: dict = None
    auth_servers: list = None
    roam_mode: str = None
    rateset: dict = None

    def __init__(self, ssid: str, site_id: str, api: API, wlan_config: dict, *args, **kwargs):
        """Initializes the Mist WLAN instance

        Initialize the following object attributes:
            ssid: a str defining the ssid (or name) of the WLAN profile
            site_id: a str defining the site_id if the site already exists on the Mist cloud
            api: a API object that contains all the necessary methos to perform API calls
            config: a Config object containing the content of the config file

        This method validates if the WLAN already exists on the Mist cloud based on its name
        If it does, the local attributes are configued via the _does_exist_on_cloud methd.
        If it doesn not, this function configured the attributes using the data provided in the config file
        """
        logger.debug("Initialiazing a Mist WLAN")
        self.ssid = ssid
        self.site_id = site_id
        self.api = api
        if self._does_exist_on_cloud() == False:
            self.band = wlan_config['band'] if 'band' in wlan_config else None
            self.interface = wlan_config['interface'] if 'interface' in wlan_config else None
            self.hostname_ie = wlan_config['hostname_ie'] if 'hostname_ie' in wlan_config else None
            self.roam_mode = wlan_config['roam_mode'] if 'roam_mode' in wlan_config else None
            self.rateset = wlan_config['rateset'] if 'rateset' in wlan_config else None
            if 'auth' in wlan_config:
                if wlan_config['auth']['type'] == "psk":
                    self._validate_psk_configuration(wlan_config)
                elif wlan_config['auth']['type'] == "eap":
                    self._validate_dot1x_configuration(wlan_config)

    def _validate_psk_configuration(self, wlan_config: dict):
        """Validates the PSK configurations

        This method validates that the proper information if provided in the confi file related to PSK Authentication
        First if validates that the password is defined in the configuration file
        Then it validates that the password is not empty. If the password is empty, in the configuration file, it raises
        an exception.
        """
        try:
            self.auth = wlan_config['auth']
            if wlan_config['auth']['psk'] == "":
                logger.warning(
                    "Authentication Type: PSK\tWARNING:The password strength is not strong enough. Please use a longer and more complexe PSK.")
        except:
            raise Exception(
                "Authentication Type: PSK\tERROR:No password defined")

    def _validate_dot1x_configuration(self, wlan_config: dict):
        """Validates the 802.1X-EAP configurations

        This method validates that the proper information if provided in the confi file related to EAP Authentication
        If validates that Authentication Servers have been provided in the configuration file.
            If it is not the case, it raises an exception
        """
        try:
            if wlan_config['auth']['type'] == 'eap':
                if 'auth_servers' in wlan_config:
                    if not wlan_config['auth_servers']:
                        raise Exception(
                            "Authentication Type: EAP\tERROR: At least one RADIUS authentication server must be defined in your configuration file")
                    self.auth = wlan_config['auth']
                    self.auth_servers = wlan_config['auth_servers']
        except:
            raise Exception(
                "Authentication Type: EAP\tERROR: At least one RADIUS authentication server must be defined in your configuration file")

    def _does_exist_on_cloud(self) -> bool:
        """Validate if a WLAN profile already exists on the Mist cloud

        This function validates if a WLAN already on the Mist Cloud
        It sends the following GET API call to retreive all wlans part of an Site:
            GET https://api.mist.com/api/v1/sites/:site_id/wlans

        If the WLAN exists on the cloud, attributes are configured based on how the Wlan
        is configured on the cloud

        Returns:
            Bool: True if the wlans exists and False is it does not exist
        """
        wlans = self.api.get(f"sites/{self.site_id}/wlans")
        for wlan in wlans:
            if wlan['ssid'] == self.ssid:
                self.wlan_id = wlan['id']
                self.band = wlan['band']
                self.interface = wlan['interface']
                self.hostname_ie = wlan['hostname_ie']
                self.roam_mode = wlan['roam_mode'] if 'roam_mode' in wlan else None
                self.auth = wlan['auth']
                self.auth_servers = wlan['auth_servers'] if 'auth_servers' in wlan else None
                self.rateset = wlan['rateset'] if 'rateset' in wlan else None
                return True
        return False

    def create(self) -> dict:
        """Creates a new WLAN on the Mist Cloud

        This function create a new Mist WLAN within a Mist Site if the WLAN doesn't exists
        The following WLAN settings are configured:
            - ssid
            - band
            - interface
            - hostname_ie (enable or disable)
            - roam_mode (OKC, 11r, None)
            - auth (open, psk, eap)
            - auth_servers (if eap auth is used)
            - rateset (data rates)

        These values are being extracted from the configuration file.

        Returns:
            response_new_wlan: a Dict containing the content of the JSON POST reply sent by the Mist Cloud
        """
        logger.info(f"Creating WLAN:\t{self.ssid}")
        wlan_body = {}
        wlan_body['enabled'] = "true"
        wlan_body['ssid'] = self.ssid
        wlan_body['band'] = self.band if 'band' in self.__dict__ else None
        wlan_body['interface'] = self.interface if 'interface' in self.__dict__ else None
        wlan_body['hostname_ie'] = self.hostname_ie if 'hostname_ie' in self.__dict__ else None
        wlan_body['roam_mode'] = self.roam_mode if 'roam_mode' in self.__dict__ else None
        wlan_body['auth'] = self.auth if 'auth' in self.__dict__ else None
        wlan_body['auth_servers'] = self.auth_servers if 'auth_servers' in self.__dict__ else None
        wlan_body['rateset'] = self.rateset if 'rateset' in self.__dict__ else None

        if self._does_exist_on_cloud() == False:
            try:
                response_new_wlan = self.api.post(f"sites/{self.site_id}/wlans", wlan_body)
            except Exception:
                raise
            self.wlan_id = response_new_wlan['id']
            logger.info(f"WLAN created:\tNAME:{self.ssid}\tID:{self.wlan_id}")
            return response_new_wlan
        else:
            logger.info(f"WLAN already exists\tID:{self.wlan_id}")

    def delete(self) -> bool:
        """Delete a WLAN on the Mist Cloud

        Deletes a WLAN on the Mist cloud if the WLAN currently exisits. The function first checks
        if the WLAN currently exists on the Mist cloud or not.
        If the WLAN exists on the Mist cloud it send the following DELETE API call to delete the WLAN:
            DELETE https://api.mist.com/api/v1/sites/:site_id/wlans/:wlan_id

        Returns:
            bool: True if the WLAN is deleted successful, False if it is not deleted
        """
        logger.info(f"Deleting WLAN {self.ssid}")
        if self._does_exist_on_cloud():
            try:
                response_delete = self.api.delete(f"sites/{self.site_id}/wlans/{self.wlan_id}")
            except Exception:
                raise
            log = f"WLAN deleted\tID:{self.wlan_id}" if response_delete else f"WLAN not deleted\tID:{self.wlan_id}"
            logger.info(log)
            self.__delete__()
            return response_delete
        else:
            logger.error(f"WLAN was NOT deleted\tREASON: WLAN doesn't currently exist on Mist Cloud")
            return False

    def __delete__(self):
        """Deletes an instance of this WLAN class
        """
        logger.debug(f"Deleting Mist WLAN Instance\tNAME:{self.ssid}\tID:{self.wlan_id}")



from semfio_mist.logger import logger, logger_engine
from semfio_mist.config import Config
from semfio_mist.mist_api import API


class Site:
    """Mist Site Object
    """

    site_id: str = None
    name: str
    api: API
    org_id: str
    timezone: str = None
    country_code: str = None
    rftemplate_id: str = None
    secpolicy_id: str = None
    alarmtemplate_id: str = None
    lat: float = None
    lng: float = None
    sitegroup_ids: [str] = None
    address: str = None

    def __init__(self, name: str, api: API, config: Config, site_id: str = None, *args, **kwargs):
        """Initializes the Mist Site instance

        Initialize the following object attributes:
            name: a str defining the name of the site (Note: this class does not allow to have
                    2 sites with the same name)
            org_id: a str defining the organization ID (coming from the API Class)
            api: a API object that contains all the necessary methos to perform API calls
            config: a Config object containing the content of the config file
            site_id: a str defining the site ID (for future use)

        This function validates if the site already exists on the Mist Cloud based on its name
        If it does, the local attributes are configued via the _does_exist_on_cloud method.
        If it does not, this function configures the attributes using the data of the config file
        """
        logger.debug("Initialiazing a Mist Site")
        self.name = name
        self.org_id = api.org_id
        self.api = api
        if self._does_exist_on_cloud() == False:
            self.timezone = config.data['site']['timezone'] if 'timezone' in config.data['site'] else None
            self.country_code = config.data['site']['country_code'] if 'country_code' in config.data['site'] else None
            self.address = config.data['site']['address'] if 'address' in config.data['site'] else None
            self.lat = config.data['site']['lat'] if 'lat' in config.data['site'] else None
            self.lng = config.data['site']['lng'] if 'lng' in config.data['site'] else None

    def _does_exist_on_cloud(self) -> bool:
        """Validate if a site already exists on the Mist cloud

        This function validates if a site already on the Mist Cloud
        It sends the following GET API call to retreive all sites part of an Organization:
            GET https://api.mist.com/api/v1/orgs/:org_id

        If the site exists on the cloud, this functions uses the JSON coming from the Mist Cloud
        to configure the following instance's attributes based on the current configuration of the site:
            - Site ID
            - Timezone
            - Country code
            - Address
            - Latitude
            - Longitude

        Returns:
            Bool: True if the site exists and Fals is it does not exist
        """
        sites = self.api.get(f"orgs/{self.org_id}/sites")
        for site in sites:
            if site['name'] == self.name:
                self.site_id = site['id']
                self.timezone = site['timezone'] if 'timezone' in site else None
                self.country_code = site['country_code'] if 'country_code' in site else None
                self.address = site['address'] if 'address' in site else None
                self.lat = site['latlng']['lat'] if 'latlng' in site else None
                self.lng = site['latlng']['lng'] if 'latlng' in site else None
                return True
        return False

    def create(self) -> dict:
        """Creates a new site on the Mist Cloud

        This function create a new Mist Site within a Mist Organization if the site doesn't exists
        The following site settings are configured:
            - name
            - timezone
            - country code
            - address
            - Latitude
            - Longitude

        These values are being extracted from the configuration file.

        Returns:
            response_new_site: a Dict containing the content of the JSON POST reply sent by the Mist Cloud

        """
        logger.info(f"Creating site:\t{self.name}")
        site_body = {}
        site_body['name'] = self.name
        if 'timezone' in self.__dict__:
            site_body['timezone'] = self.timezone
        if 'country_code' in self.__dict__:
            site_body['country_code'] = self.country_code
        if 'address' in self.__dict__:
            site_body['address'] = self.address
        if 'lat' in self.__dict__ and 'lng' in self.__dict__:
            site_body['latlng'] = {'lat': self.lat, 'lng': self.lng}

        if self._does_exist_on_cloud() == False:
            try:
                response_new_site = self.api.post(f"orgs/{self.org_id}/sites", site_body)
            except Exception:
                raise
            self.site_id = response_new_site['id']
            logger.info(f"Site created:\tNAME:{self.name}\tID:{self.site_id}")
            return response_new_site
        else:
            logger.info(f"Site already exists\tID:{self.site_id}")

    def configure_persist_config_on_device(self, config_persistence_enable: bool) -> dict:
        """Configure the AP Config Persistence feature

        Updates the configurations of a site to enable the following feature:
            AP Config Persistence (or 'persist_config_on_device')

        Args:
            config_persistence_enable: bool defining to either enable or disable the feature

        Returns:
            response_configure: a Dict containing the content of the JSON PUT reply sent by the Mist Cloud
        """
        logger.debug(f"Configuring AP Config Persitence for this site: {self.name}")
        data_put = {}
        data_put['persist_config_on_device'] = config_persistence_enable
        try:
            response_configure = self.api.put(f"sites/{self.site_id}/setting", data_put)
        except Exception:
            raise
        self.config_persistence_enable = config_persistence_enable
        logger.info(f"AP Config Persistence configured\tSITE:{self.name}")
        return response_configure

    def delete(self) -> bool:
        """Delete a site on the Mist Cloud

        Deletes a Site on the Mist cloud if the site currently exisits. The function first checks
        if the site currently exists on the Mist cloud or not.
        If the site exists on the Mist cloud it send the following DELETE API call to delete the site:
            DELETE https://api.mist.com/api/v1/sites/:site_id

        Returns:
            bool: True if the site is deleted successful, False if it is not deleted
        """
        logger.info(f"Deleting site {self.site_id}")
        if self._does_exist_on_cloud():
            try:
                response_delete = self.api.delete(f"sites/{self.site_id}")
            except Exception:
                raise
            log = f"Site deleted\tID:{self.site_id}" if response_delete else f"Site not deleted\tID:{self.site_id}"
            logger.info(log)
            self.__delete__()
            return response_delete
        else:
            logger.error(f"Site was NOT deleted\tREASON: Site doesn't currently exist on Mist Cloud")
            return False

    def __delete__(self):
        """Deletes an instance of this Site class
        """
        logger.debug(f"Deleting Mist Site Instance\tNAME:{self.name}\tID:{self.site_id}")



import json
from semfio_mist.logger import logger, logger_engine
from semfio_mist.config import Config
from semfio_mist.mist_api import API


class AP:
    """Mist AP Object

    This class is used to perform Mist AP operations. It is mainly used to initially configure
    Access Points as well as adjust their radio configurations.

    Attributes:
        mac: str defining the MAC address of the AP
        name: str defining the name of the AP
        site_id: str defining the ID of the site the AP belongs to
        api: API instance
        config: Config instance containing the content of the config file(s)
        id: str defining the ID of the Mist AP on the Mist Cloud once it has been claimed
        site_name: str defining the name of the site the AP belongs to
        model: str defining the model of the AP
        serial: str defining the serial number of the AP
        claim_code: str defining the claim code of the AP
        height: str defining the installation height of the AP
        orientation: str defining the orientation of an AP
        map: dict defining the map ID of the map the AP is placed on as well as the coordinates of its location
        radio_configs: dict defining both 2.4Ghz and 5Ghz radio configurations of the AP

    Methods:
        __init__(self, mac: str, site_id: str, api: API, config: Config, *args, **kwargs: dict):
            Used to initialize an AP instance and set some of the key Attributes
        _has_been_claimed(self, org_id) -> bool: Validate if an AP is already in an Organization inventory
        _does_belong_to_site(self) -> bool: Validate is an AP belongs to a site
        claim(self, org_id): Claim a Mist AP to an organization
        provision(self, ap_name): Assign an AP to a Mist Site
        configure_radios(self): Configure both 2.4GHz and 5GHz radio settings of a Mist AP
        unassign(self, org_id): Unassign an AP from a Mist Site. The AP will still be part of the Organization inventory
        release(self, org_id): Remove the AP from an Organization inventory
    """

    mac: str
    name: str
    site_id: str
    api: API
    config: Config
    id: str = None
    site_name: str = None
    model: str = None
    serial: str = None
    claim_code: str = None
    height: str = None
    orientation: str = None
    map: dict = {}
    radio_configs = {}

    def __init__(self, mac: str, site_id: str, api: API, config: Config, *args, **kwargs: dict):
        """Initializes a Mist AP instance

        Initialize the following object attributes:
            mac: a str defining the mac address of the AP using the value passed as an argument
            name: a str defining the name of the AP. It is retreived from the configuration file
            site_id: a str defining the site ID the AP belongs to using the value passed as an argument
            api: a API object that contains all the necessary methos to perform API calls
            config: a Config object containing the content of the config file(s)
            site_name: a str defining the name of the site the AP belongs to. It is retreived from the configuration file
            claim_code: a str used to claim an AP to a Mist organization. It is retreived from the configuration file
            height: a str used to defining the installation height of an AP. It is retreived from the configuration file
            orientation: a str used to define the orientation of an AP. It is retreived from the configuration file
            radio_configs: a dict containing the radio configurations. It is retreived from the configuration file

        Args:
            mac: MAC address of the AP
            site_id: Site ID of the site the AP belongs to, or Site ID of the site it will belong to
            api: API object that contains all the necessary methos to perform API calls
            config: a Config object containing the content of the config file(s)
        """
        logger.debug("Initialiazing a Mist AP")
        self.mac = mac
        self.name = config.data['ap']['name'] if 'name' in config.data['ap'] else None
        self.site_id = site_id
        self.api = api
        self.config = config
        self.site_name = config.data['site']['name']
        self.claim_code = config.data['ap']['claim_code'] if 'claim_code' in config.data['ap'] else None
        self.height = config.data['ap']['height'] if 'height' in config.data['ap'] else None
        self.orientation = config.data['ap']['orientation'] if 'orientation' in config.data['ap'] else None
        self.radio_configs['radio_config'] = {}

    def _has_been_claimed(self, org_id) -> bool:
        """Validates if an AP has already been claimed to an organization

        This functions validates if an AP has been claimed to an Organization by looking to see
        if it is part of the organization inventory.
        It sends the following GET API call to retreive the device inventory of the organization:
            GET https://api.mist.com/installer/orgs/:org_id/devices

        If the AP is part of this inventory (validation made against the MAC address of the AP),
        the function returns True. If not, it returns False.

        Returns:
            Bool: True if the AP has been claimed, False if not
        """
        logger.debug(
            f"Validating if AP {self.mac} has already been claim to Organization {org_id}")
        devices = self.api.get(f"installer/orgs/{org_id}/devices")
        for device in devices:
            if device['mac'] == self.mac:
                return True
        return False

    def _does_belong_to_site(self) -> bool:
        """Validates if an AP is part of a Mist site

        This function check if an AP is already assigned to a site based on
        the AP MAC Address

        It sends the following API Call to retrieve all the AP part of a site:
            GET https://api.mist.com/v1/sites/:site_id/devices

        If the AP is part of the site, this function set the following Attributes
        based on the API response content:
            id: str device id
            serial: str serial number of the AP
            model: str model name of the AP
            map['id']: str map ID if the AP is located on a map

        Returns:
            Bool: True if the AP is part of the site, False if it is not
        """
        logger.debug(f"Validating if a AP {self.mac} belongs to Site {self.site_id}")
        devices = self.api.get(f"sites/{self.site_id}/devices")
        for device in devices:
            if (device['type'] == "ap") and (device['mac'] == self.mac):
                self.id = device['id']
                self.serial = device['serial']
                self.model = device['model']
                self.map['id'] = device['map_id'] if 'map_id' in device else None
                return True
        return False

    def claim(self, org_id: str):
        """Claim an AP to an organization

        This function will claim an AP to an Organization. This is typically the first thing
        you would do taking the AP outside the box. The function checks if the AP has not
        been claimed yet first.

        The claim code of the AP is required for this operation. This claim code can be found
        at the back of the AP.

        It sends the following POST API Call to the Mist Cloud:
            POST https://api.mist.com/orgs/:org_id/inventory

        The claim code is sent as the POST request body in the following fashon:
            ["<claim_code>"]

        Once the AP has been claimed to the organization, the following attributes are configured:
            id: ID of the Mist AP object on the Mist cloud
            serial: serial number of the AP
            model: model of the AP

        Args:
            org_id: ID of the Organization
        """
        logger.debug(f"Claiming AP {self.mac} to Organization {org_id}")
        if self._has_been_claimed(self.api.org_id) == False:
            if self.claim_code != None:
                claim_body = []
                claim_body.append(f"{self.claim_code}")
                claim_response = self.api.post(f"orgs/{org_id}/inventory", claim_body)
                self.id = claim_response['inventory_added'][0]['id']
                self.serial = claim_response['inventory_added'][0]['serial']
                self.model = claim_response['inventory_added'][0]['model']
            else:
                raise Exception(f"No claim code has been provided to claim this AP.")

    def provision(self, ap_name: str):
        """Assign an AP to a Mist Site

        This functions assign a Mist AP to a Mist Site.
        It sends a PUT API Call to the Mist Cloud:
            PUT https://api.mist/com/installer/orgs/:org_id/devices/:ap_mac"

        The following data is configured:
            AP Name
            MAP ID (if the AP needs to be placed on a map)
            AP height (if this information is provided in the configuration files)
            AP orientation (if this information is provided in the configuration files)

        Once the AP is assigned to the site, the following attributes are updated:
            id: ID of the Mist AP object on the Mist cloud
            serial: serial number of the AP
            model: model of the AP

        Args:
            ap_name: name of the AP
        """
        logger.debug(f"Provisioning AP {self.mac}")
        ap_provision = {}
        ap_provision['name'] = ap_name
        ap_provision['site_id'] = self.site_id
        if (self.map) and ('id' in self.map):
            ap_provision['map_id'] = self.map['id']
        if self.height:
            ap_provision['height'] = self.height
        if self.orientation:
            ap_provision['orientation'] = self.orientation

        if self._does_belong_to_site() == False:
            try:
                response_provision = self.api.put(
                    f"installer/orgs/{self.api.org_id}/devices/{self.mac}", ap_provision)
            except Exception:
                raise
            self.id = response_provision['id']
            self.serial = response_provision['serial']
            self.model = response_provision['model']
        else:
            logger.info(f"AP already belongs to site\tID:{self.site_id}")

    def configure_radios(self):
        """Confiure Radio Settings of an AP

        This functions configure the configurations of both radios of an AP. Both of them
        are configured at the same time.

        It configures the following settings of the 2.4Ghz radio:
            Channel
            Tx Power

        It configures the following settings of the 5GHz radio:
            Channel
            Channel Bandwidth
            Tx Power

        It sends a PUT API Call to the Mist Cloud:
            PUT https://api.mist.com/sites/:site_id/devices/:device_id
        """
        logger.debug(f"Configuring AP Radio Settings")
        if self._does_belong_to_site():
            self.radio_configs['radio_config']['band_24'] = {}
            self.radio_configs['radio_config']['band_24']['power'] = self.config.data['ap']['24']['power']
            self.radio_configs['radio_config']['band_24']['channel'] = self.config.data['ap']['24']['channel']
            self.radio_configs['radio_config']['band_5'] = {}
            self.radio_configs['radio_config']['band_5']['power'] = self.config.data['ap']['5']['power']
            self.radio_configs['radio_config']['band_5']['bandwidth'] = self.config.data['ap']['5']['bandwidth']
            self.radio_configs['radio_config']['band_5']['channel'] = self.config.data['ap']['5']['channel']
            radio_configs_response = self.api.put(
                f"sites/{self.site_id}/devices/{self.id}", self.radio_configs)
            logger.info(f"AP: {self.name}\t2.4GHz Radio Configured:\
                        CHANNEL:{radio_configs_response['radio_config']['band_24']['channel']}\
                        POWER:{radio_configs_response['radio_config']['band_24']['channel']}")
            logger.info(f"AP: {self.name}\t5GHz Radio Configured:\
                        CHANNEL:{radio_configs_response['radio_config']['band_5']['channel']}\
                        BANDWIDTH:{radio_configs_response['radio_config']['band_5']['bandwidth']}\
                        POWER:{radio_configs_response['radio_config']['band_5']['power']}")
        else:
            raise Exception(f"AP {self.name} does not belong to a site.")

    def unassign(self, org_id: str):
        """Unassign an AP from any site

        This function unassign an AP from a site. The AP stays in the inventory of the Organization
        and can be assign to other sites.

        It sends a PUT API Call to the Mist cloud:
            PUT https://api.mist.com/orgs/:org_id/inventory

        The body of the PUT API call is organized as follow:
            {
                "op": "unassign",
                "macs": [
                    "<ap_mac_address>"
                ]
            }

        Once the call is made, the site_id attribute and site_names are updated to None
        """
        logger.debug(f"Unassigning AP {self.mac} from site {self.site_id}")
        if self._does_belong_to_site():
            unassign_body = {}
            unassign_body['op'] = "unassign"
            unassign_body['macs'] = []
            unassign_body['macs'].append(f"{self.mac}")
            unassign_response = self.api.put(f"orgs/{org_id}/inventory", unassign_body)
            logger.debug(unassign_response)
            if unassign_response['success'][0] == self.mac:
                logger.info(f"AP {self.mac} has been unassigned from site {self.site_id}")
                self.site_id = None
                self.site_name = None
            else:
                logger.error(f"AP has not been unassigned from site")

    def release(self, org_id: str):
        """Release an AP from any site

        This function deletes an AP from an organization inventory. It first checks if
        the AP is part of the inventory.

        It sends a PUT API Call to the Mist Cloud:
            PUT https://api.mist.com/orgs/:org_id/inventory

        The body of the PUT API call is organized as follow:
            {
                "op": "delete",
                "macs": [
                    "<ap_mac_address>"
                ]
            }

        Once the call is made, the site_id attribute and site_names are updated to None
        """
        logger.debug(f"Realising AP {self.mac} from organization {org_id}")
        if self._has_been_claimed(org_id):
            release_body = {}
            release_body['op'] = "delete"
            release_body['macs'] = []
            release_body['macs'].append(f"{self.mac}")
            release_response = self.api.put(f"orgs/{org_id}/inventory", release_body)
            logger.debug(release_response)
            if release_response['success'][0] == self.mac:
                logger.info(f"AP {self.mac} has been released from organization {org_id}")
                self.site_id = None
                self.site_name = None
            else:
                logger.error(f"AP has not been released from organization")
        else:
            logger.info(f"AP {self.mac} is currently not part of organization {org_id}")



import json
from semfio_mist.logger import logger


class Config:
    """Config Object

    User configuration information will be loaded into this Object
    Make sure that all the desired data is added to a JSON configuration file
    before running the script.

    Attributes:
        filename: str script configuration filename wirtten in JSON
        data: A dict containing the content of the filename
    """

    def __init__(self, filename, *args, **kwargs):
        """Inits Config class

        Loads the content of the JSON configuration file into the data attribute (dict)

        Args:
            filename: JSON configuration file
        """
        self.filename = filename

        try:
            with open(self.filename) as config_file:
                file_content = config_file.read()
        except Exception as e:
            logger.error(f"Unable to open the following configuration file: {filename}")
            raise e

        self.data = json.loads(file_content)



# __init__.py

# Version of the semfio-mist package
__version__ = "0.0.3"

import semfio_mist
from .config import Config
from .logger import logger, Logger_Engine
from .mist_api import API
from .mist_site import Site
from .mist_wlan import WLAN
from .mist_ap import AP



import requests
import os
import json
from semfio_mist.config import Config
from semfio_mist.logger import logger


class Token:
    """Mist Token Object

    This class allows us to manage and handle the token(s) used to make calls to the
    Mist API cloud.

    It assumes that the your MIST API MASTER TOKEN is store in an environment variable
    on your system.

    * MACOS
    To create an environmental variable on macOS, run this command:
        export MIST_TOKEN='<your-token-here>'

    Note that this variable will be lost when you close the terminal. In order to create
    an environment variable that will be saved, open the following file:
        > vi ~/.zshrc
            > add the following line: export MIST_TOKEN='<your-token-here>'
                > save the .zshrc file
                    > run this command: source ~/.zshrc

    In order to validate that the variable is defined, run this command:
        export | grep MIST_TOKEN


    * WINDOWS
    To create the user env var, use the following command from a command window on a
    Windows 10 machine:
        setx MIST_TOKEN "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"

    To verify the env var has been correctly created, open a new command windows and
    type the following command to verify that the env var is now gloablly available to
    the user account being used:
        echo %MIST_TOKEN%

    Note that the env var is only available in a NEW command window. The env var is
    now permanently avaialble to all processes running under the current user account.
    The env var will not be available to other users on the same machine.

    To remove the env var value, set the env var with a blank value in a command window
    (the env var will still exist, but will have no value):
        setx MIST_TOKEN ""

    Or, alternatively it may be deleted via the Windows 10 GUI environmental variable
    editing tool:
        > Start
            > Control Panel
                > System & Security
                    > System
                        > Advanced System Settings
                            > Environmental Variables (User section)

    This Class gives use the ability to create a temporary token that could be used to perform
    all API calls required by a specific script or program.

    The MASTER TOKEN will only be used to create and delete the temporary token.

    Attributes:
        MASTER_TOKEN: str for the KEY main Mist API Token
        tmp_token_id: str for the ID of the temporary token
        tmp_token_key: str for the KEY of the temporary token
    """

    def __init__(self, config: Config):
        """Initialized the Mist Token object

        Set the MASTER_TOKEN variable:
            - Retrieve the environment variable called MIST_TOKEN
            - Make sure the this environ is set on your system
                - Run this command on MacOS: export MIST_TOKEN='<your-token-here>'
        """
        try:
            self.MASTER_TOKEN = os.environ['MIST_TOKEN'] if 'MIST_TOKEN' in os.environ else config.data['token']
        except Exception:
            raise

    def get_tmp_token(self, session: requests.Session):
        """Creates a Mist token 'on the fly' to be used witin a specific script.

        The following API call is made to the Mist Cloud to create a new token:
            POST https://api.mist.com/api/v1/self/apitokens

        Set the following class variables:
         - tmp_token_key: temporary token KEY received from Mist Cloud after creation
         - tmp_token_id: temporary token ID received from Mist Cloud after creation
        """
        api_url = "https://api.mist.com/api/v1/self/apitokens"
        headers = {"Content-Type": "application/json",
                   "Authorization": f"Token {self.MASTER_TOKEN}"}
        try:
            response = session.post(api_url, data={}, headers=headers)
            if response.status_code == 200:
                token = json.loads(response.text)
                self.tmp_token_key = token['key']
                self.tmp_token_id = token['id']
                logger.debug(
                    f"Temporary Token created: {self.tmp_token_key}\tID: {self.tmp_token_id}")
            else:
                raise ValueError(
                    f"Error connecting to Mist API!\tRESPONSE:'{response.status_code} - {response.text}'")
        except Exception:
            raise

    def delete_tmp_token(self, session: requests.Session):
        """Deletes the temporary token

        Deletes the temporary Mist Token stored in self.tmp_token_key
        using its ID stored in self.tmp_token_id

        The following API call is made to the Mist Cloud to delete the token:
            DELETE https://api.mist.com/api/v1/self/apitokens/:token_id
        """
        if "tmp_token_id" in self.__dict__:
            api_url = f"https://api.mist.com/api/v1/self/apitokens/{self.tmp_token_id}"
            headers = {"Content-Type": "application/json",
                       "Authorization": f"Token {self.MASTER_TOKEN}"}
            try:
                response = session.delete(api_url, headers=headers)
                if response.status_code == 200:
                    logger.debug(f"Token deleted\tID: {self.tmp_token_id}")
                else:
                    raise ValueError(
                        f"Error connecting to Mist API!\tRESPONSE:'{response.status_code} - {response.text}'")
            except Exception:
                raise
        else:
            raise ValueError(f"tmp_token_id does not exits.")


class API:
    """ Mist API Object

    Enables us to perform API calls to the Mist cloud

    Attributes:
        session: requests.Session object used to open a TCP connection for multiple API calls
        mist_cloud_url: str use to define the URL of the Mist cloud used for the API calls
        org_id: str ID of the Mist organization used within the API calls
        _mist_token: Token object containing the token used for the API calls
        _headers: dict containing the header used for the API calls

    """

    mist_cloud_url = "https://api.mist.com/api/v1/"
    session: requests.Session
    org_id: str
    _mist_token: Token
    _headers: dict = {}

    def __init__(self, config: Config, cloud: str = "", *args, **kwargs):
        """Initialized the Mist API object.

        Initialize the following object attributes:
            session: a new request.session is created and store in this attribute
            mist_cloud_url: if the cloud arg is set to EU, this attribute is adjusted accordingly
            _mist_token: a new Token object is created and a temporary token is requested to be used by this program
            org_id: retreived from MIST_ORG environement variable if exists or from the JSON config file otherwise
            _headers: dict containing the temporary token for authorization

        At the end of this Initialization, we have all the elements ready to make API calls
        towards the Mist cloud.

        Args:
            config: Config object containing the content of the config file
            cloud: str (Optional), "" to use US Mist cloud, "EU" to use European Mist Cloud
        """
        logger.debug("Initialiazing connection to the Mist API")
        self.session = requests.Session()
        self._mist_token = Token(config)
        self._mist_token.get_tmp_token(self.session)
        self.org_id = os.environ['MIST_ORG'] if 'MIST_ORG' in os.environ else config.data['org_id']
        self._headers = {"Content-Type": "application/json",
                         "Authorization": f"Token {self._mist_token.tmp_token_key}"}
        if cloud == "EU":
            self.mist_cloud_url = "https://api.eu.mist.com/api/v1/"

    def _verify_response(self, response: requests.Response) -> dict:
        """Verify the Status of the API GET or POST call

        If the API call was successful, it converts the JSON reply into a python dictionary
        If the API call was unsucessful, it logs an error message with the details

        Returns:
            reponse_text: a Dict containing the content of the JSON reply sent by Mist Cloud
        """
        response_text = json.loads(response.text)
        logger.debug(f"API CAL Status Code: {response.status_code}")
        if response.status_code >= 400:
            logger.error(
                f"API Call error {response.status_code}: {response_text}")
        else:
            return response_text

    def _verify_delete_response(self, response: requests.Response) -> bool:
        """Verify the status of a DELETE API Call

        If the API call was successful, it returns True
        If the API call was unsucessful, it logs an error and returns False

        Returns:
            Bool: stating the status of the DELETE API CALL
        """
        response_text = json.loads(response.text)
        logger.debug(f"API CALL Status Code: {response.status_code}")
        if response.status_code != 200:
            logger.error(
                f"API Call error {response.status_code}: {response_text['message']}")
            return False
        else:
            return True

    def get(self, call_url: str) -> dict:
        """Performs a GET API call to the Mist cloud

        Args:
            call_url: str defining the second part of the API api_url
                Example: "org/:org_id/sites" in "https://api.mist.com/api/v1/org/:org_id/sites"

        Returns:
            A dict containing the content of the response
        """
        call_url = self.mist_cloud_url + call_url
        try:
            logger.debug(f"Sending API GET CALL: {call_url}")
            response = self.session.get(call_url, headers=self._headers)
        except Exception:
            raise
        return self._verify_response(response)

    def post(self, call_url: str, body: dict) -> dict:
        """Performs a POST API call to the Mist cloud

        Args:
            call_url: str defining the second part of the API api_url
                Example: "org/:org_id/sites" in "https://api.mist.com/api/v1/org/:org_id/sites"
            body: dict containing the data to be sent along with the POST API Call

        Returns:
            A dict containing the content of the response
        """
        call_url = self.mist_cloud_url + call_url
        try:
            logger.debug(f"Sending API POST CALL: {call_url}")
            response = self.session.post(call_url, data=json.dumps(body), headers=self._headers)
        except Exception:
            raise
        return self._verify_response(response)

    def put(self, call_url: str, body: dict) -> dict:
        """Performs a PUT API call to the Mist cloud

        Args:
            call_url: str defining the second part of the API api_url
                Example: "org/:org_id/sites" in "https://api.mist.com/api/v1/org/:org_id/sites"
            body: dict containing the data to be sent along with the PUT API Call

        Returns:
            A dict containing the content of the response
        """
        call_url = self.mist_cloud_url + call_url
        try:
            logger.debug(f"Sending API PUT CALL: {call_url}")
            response = self.session.put(call_url, data=json.dumps(body), headers=self._headers)
        except Exception:
            raise
        return self._verify_response(response)

    def delete(self, call_url: str) -> bool:
        """Performs a DELETE API call to the Mist Cloud

        Args:
            call_url: str defining the second part of the API api_url
                Example: "org/:org_id/sites" in "https://api.mist.com/api/v1/org/:org_id/sites"

        Returns:
            Bool: indicating if the element was deleted
        """
        call_url = self.mist_cloud_url + call_url
        try:
            logger.debug(f"Sending API DELETE CALL: {call_url}")
            response = self.session.delete(call_url, headers=self._headers)
        except Exception:
            raise
        return self._verify_delete_response(response)

    def __exit__(self):
        self._mist_token.delete_tmp_token(self.session)
        self.session.close()
        logger.debug("Connection to the Mist API closed")
